/*
 * This file is part of the Code::Blocks IDE and licensed under the GNU Lesser General Public License, version 3
 * http://www.gnu.org/licenses/lgpl-3.0.html
 */

#ifndef SCRIPTSECURITYWARNINGDLG_H
#define SCRIPTSECURITYWARNINGDLG_H

#include "scrollingdialog.h"

enum ScriptSecurityResponse
{
    ssrAllow = 0,
    ssrAllowAll,
    ssrDeny,
    ssrTrust,
    ssrTrustPermanently
};

class ScriptSecurityWarningDlg : public wxScrollingDialog
{
    public:
        ScriptSecurityWarningDlg(wxWindow* parent, const wxString& operation, const wxString& command);
        ~ScriptSecurityWarningDlg() override;

        ScriptSecurityResponse GetResponse();
        void EndModal(int retCode) override;
    protected:
    private:
};



#endif // SCRIPTSECURITYWARNINGDLG_H
