/** \file wxsproperties.h
 *
 * This file includes all files needed for operating on properties, including
 * all standard properties and all extended ones
 */

// Main files
#include "wxsproperty.h"
#include "wxspropertycontainer.h"
#include "wxspropertygridmanager.h"
#include "wxspropertystream.h"
#include "wxsquickpropspanel.h"

// Base properties
#include "wxsarraystringproperty.h"
#include "wxsboolproperty.h"
#include "wxscustomeditorproperty.h"
#include "wxsemptyproperty.h"
#include "wxsenumproperty.h"
#include "wxsflagsproperty.h"
#include "wxsfloatproperty.h"
#include "wxslongproperty.h"
#include "wxsstringproperty.h"
#include "wxseditenumproperty.h"
