/*
 * This file is part of the Code::Blocks IDE and licensed under the GNU Lesser General Public License, version 3
 * http://www.gnu.org/licenses/lgpl-3.0.html
 */

/** Code::Blocks SDK precompiled headers support for internal usage.
  * This is precompiled with -DEXPORT_LIB -DEXPORT_EVENTS
  */

#ifndef SDK_PRECOMP_H
#define SDK_PRECOMP_H

#include "sdk_common.h"

#endif // SDK_PRECOMP_H
